@@ -1,7 +1,7 @@
 package AnySan::Provider::Slack;
 use strict;
 use warnings;
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use base 'AnySan::Provider';
 our @EXPORT = qw(slack);
@@ -29,43 +29,50 @@ sub slack {
         ], sub {});
     }
 
-    # get auth info
-    $self->_call('auth.test', [], sub {
-        my $res = shift;
-        $self->{authinfo} = $res;
-    });
+    $self->start;
 
-    my $rtm = AnyEvent::SlackRTM->new($config{token});
+    return $self;
+}
+
+sub metadata { shift->{rtm}->metadata }
+
+sub start {
+    my $self = shift;
+
+    my $rtm = AnyEvent::SlackRTM->new($self->{config}{token});
     $rtm->on('hello' => sub {
-        $self->{keep_alive} = AnyEvent->timer(
-            interval => 60,
-            cb => sub {
-                $rtm->ping;
-            },
-        );
     });
     $rtm->on('message' => sub {
         my ($rtm, $message) = @_;
-        my $authinfo = $self->{authinfo} or return;
-        return if $message->{subtype} && $message->{subtype} eq 'bot_message';
-        return if $message->{user} && $message->{user} eq $authinfo->{user_id};
+        my $metadata = $self->metadata or return;
+        if ($message->{subtype}) {
+            my $filter = $self->{config}{subtypes} || [];
+            return unless grep { $_ eq 'all' || $_ eq $message->{subtype} } @$filter;
+        }
         my $receive; $receive = AnySan::Receive->new(
             provider      => 'slack',
             event         => 'message',
             message       => encode_utf8($message->{text} || ''),
-            nickname      => encode_utf8($authinfo->{user} || ''),
+            nickname      => encode_utf8($metadata->{self}{name} || ''),
             from_nickname => encode_utf8($message->{user} || ''),
             attribute     => {
                 channel => $message->{channel},
+                subtype => $message->{subtype},
             },
             cb            => sub { $self->event_callback($receive, @_) },
         );
         AnySan->broadcast_message($receive);
     });
+    $rtm->on('finish' => sub {
+        # reconnect
+        undef $self->{rtm};
+        while (1) {
+            eval { $self->start };
+            last unless $@;
+        }
+    });
     $rtm->start;
     $self->{rtm} = $rtm;
-
-    return $self;
 }
 
 sub event_callback {
@@ -121,13 +128,34 @@ B<THE SOFTWARE IS ALPHA QUALITY. API MAY CHANGE WITHOUT NOTICE.>
 
   use AnySan;
   use AnySan::Provider::Slack;
-  my $slack = slack
+  my $slack = slack(
       token => 'YOUR SLACK API TOKEN',
       channels => {
           'general' => {},
-      };
+      },
+
+      as_user => 0, # post messages as bot (default)
+      # as_user => 1, # post messages as user
+
+      subtypes => [], # ignore all subtypes (default)
+      # subtypes => ['bot_message'], # receive messages from bot
+      # subtypes => ['all'], # receive all messages(bot_message, me_message, message_changed, etc)
+  );
   $slack->send_message('slack message', channel => 'C024BE91L');
 
+  AnySan->register_listener(
+      slack => {
+          event => 'message',
+          cb => sub {
+              my $receive = shift;
+              return unless $receive->message;
+              warn $receive->message;
+              warn $receive->attribute->{subtype};
+              $receive->send_reply('hogehoge');
+          },
+      },
+  );
+
 =head1 AUTHOR
 
 Ichinose Shogo E<lt>shogo82148@gmail.com E<gt>
